from abc import ABC, abstractmethod
from typing import Optional

from nhm_spider import Spider
from nhm_spider.http import Request, Response
from nhm_spider.utils.log import get_logger


class BaseDownloader(ABC):
    def __init__(self, spider: Spider):
        self.logger = get_logger(self.__class__.__name__)
        self.spider = spider
        self.headers = None
        self.timeout = self.spider.settings.get_int("REQUEST_TIMEOUT", 180)
        self.__opened: bool = False

        self.clear_cookie = self.spider.settings.get_bool("CLEAR_COOKIE", False)
        self.use_session = self.spider.settings.get_bool("USE_SESSION", True)

    async def open_downloader(self):
        self.headers = self.spider.settings.get_dict("DEFAULT_REQUEST_HEADER")
        self.__opened = True

    async def close_downloader(self):
        self.__opened = False

    @property
    def is_opened(self) -> bool:
        return self.__opened

    @abstractmethod
    async def send_request(self, request: Request) -> Optional[Response | Exception]:
        raise NotImplementedError()



from abc import ABC, abstractmethod
from copy import deepcopy
from typing import Any

from nhm_spider.exceptions import ValidationError


# todo: 待增加不同类型的字段，增加字段`类型检查`或`自动转换`功能。
#       IntegerField, StringField, FloatField, JsonField ...
#       再深入可考虑`长度检查`等
class BaseField(ABC):
    def __init__(self):
        self.__value = None

    def __set__(self, instance, value):
        self.validate(value)
        self.__value = value

    def __get__(self, instance, owner):
        return self.__value

    @abstractmethod
    def validate(self, value: Any) -> bool:
        raise NotImplemented


class Field(BaseField):
    def validate(self, value: Any) -> bool:
        return True


class IntegerField(Field):
    def validate(self, value: Any):
        if not isinstance(value, int):
            raise ValidationError(f"The value of an {self.__class__.__name__} instance must be an integer.")



import logging
from logging import INFO, Formatter, Logger, StreamHandler
from typing import Union


class LoggerManager:
    __instance = {}

    @staticmethod
    def hash(name, log_level):
        return hash((name, log_level))

    @classmethod
    def get_logger(cls, name: str = "default", log_level=None):
        """
        单例
        """
        hash_code = cls.hash(name, log_level)
        if hash_code in cls.__instance:
            return cls.__instance[hash_code]

        # 使用标准 logging.getLogger() 替代自定义 Logger
        _logger = logging.getLogger(name)

        # 避免重复添加处理器
        if not _logger.handlers:
            formatter_option = f"%(asctime)s [%(name)s] %(levelname)s: %(message)s"
            logger_formatter = Formatter(formatter_option)
            handler = StreamHandler()
            handler.setFormatter(logger_formatter)
            handler.setLevel(log_level or INFO)
            _logger.addHandler(handler)

        _logger.setLevel(log_level or INFO)
        cls.__instance[hash_code] = _logger
        return _logger

    @classmethod
    def set_log_level(cls, logger: Logger, level: Union[str, int]):
        [handler.setLevel(level) for handler in logger.handlers]
        logger.setLevel(level)


get_logger = LoggerManager.get_logger
